from datetime import timedelta

from mailman.config import config
from mailman.core.i18n import _
from mailman.interfaces.action import Action
from mailman.interfaces.styles import IStyle
from mailman.styles.base import (Announcement, BasicOperation, Bounces,
                                 Discussion, Identity, Moderation, Private,
                                 Public)
from mailman.utilities.modules import abstract_component
from public import public
from zope.interface import implementer


@abstract_component
@implementer(IStyle)
class FsmiBaseStyle(Identity, BasicOperation, Bounces, Discussion, Moderation):
    def apply(self, mailing_list):
        Identity.apply(self, mailing_list)
        BasicOperation.apply(self, mailing_list)
        Bounces.apply(self, mailing_list)
        Discussion.apply(self, mailing_list)
        Moderation.apply(self, mailing_list)

        # use sites default_language instead of 'en'
        mailing_list.preferred_language = config.mailman.default_language


@abstract_component
class FsmiPublicBaseStyle(FsmiBaseStyle, Public):
    def apply(self, mailing_list):
        super().apply(mailing_list)
        Public.apply(self, mailing_list)


@abstract_component
class FsmiInternalBaseStyle(FsmiBaseStyle, Private):
    def apply(self, mailing_list):
        super().apply(mailing_list)
        Private.apply(self, mailing_list)


@public
class FsmiPublicDiscussionStyle(FsmiPublicBaseStyle):
    name = 'fsmi-public-discussion'
    description = _('Public discussion mailing list style.')


@public
class FsmiPublicAnnouncemntStyle(FsmiPublicBaseStyle, Announcement):
    name = 'fsmi-public-announcement'
    description = _('Public announce only mailing list style.')

    def apply(self, mailing_list):
        super().apply(mailing_list)
        Announcement.apply(self, mailing_list)

        mailing_list.default_member_action = Action.hold
        mailing_list.default_nonmember_action = Action.hold
        # Low traffic list
        mailing_list.digests_enabled = False
        mailing_list.bounce_info_stale_after = timedelta(days=180)


@public
class FsmiPublicModeratedStyle(FsmiPublicBaseStyle):
    name = 'fsmi-public-moderated'
    description = _('Public moderated mailing list style.')

    def apply(self, mailing_list):
        super().apply(mailing_list)
        mailing_list.default_member_action = Action.hold
        mailing_list.default_nonmember_action = Action.hold


@public
class FsmiInternalDiscussionStyle(FsmiInternalBaseStyle):
    name = 'fsmi-internal'
    description = _('Internal discuission mailing list style.')
